# ALMA Data Reduction Script

# Calibration

thesteps = []
step_title = {0: 'Import of the ASDM',
              1: 'Fix of SYSCAL table times',
              2: 'listobs',
              3: 'A priori flagging',
              4: 'Generation and time averaging of the WVR cal table',
              5: 'Generation of the Tsys cal table',
              6: 'Generation of the antenna position cal table',
              7: 'Application of the WVR, Tsys and antpos cal tables',
              8: 'Split out science SPWs and time average',
              9: 'Listobs, clear pointing table, and save original flags',
              10: 'Initial flagging',
              11: 'Putting a model for the flux calibrator(s)',
              12: 'Save flags before bandpass cal',
              13: 'Bandpass calibration',
              14: 'Save flags before gain cal',
              15: 'Gain calibration',
              16: 'Save flags before applycal',
              17: 'Application of the bandpass and gain cal tables',
              18: 'Split out corrected column'}

if 'applyonly' not in globals(): applyonly = False
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^4.2.2', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 4.2.2')


# CALIBRATE_AMPLI: J0510+1800
# CALIBRATE_ATMOSPHERE: HL_Tau,J0423-0120,J0431+1731,J0510+1800
# CALIBRATE_BANDPASS: J0423-0120
# CALIBRATE_FLUX: J0510+1800
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: J0431+1731
# CALIBRATE_POINTING: J0423-0120
# OBSERVE_TARGET: HL_Tau

# Using reference antenna = DA51

# Import of the ASDM
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  if os.path.exists('uid___A002_X91cc20_X6d8.ms') == False:
    importasdm('uid___A002_X91cc20_X6d8', asis='Antenna Station Receiver Source CalAtmosphere CalWVR')
  if applyonly != True: es.fixForCSV2555('uid___A002_X91cc20_X6d8.ms')

# Fix of SYSCAL table times
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_X91cc20_X6d8.ms')

print "# A priori calibration"

# listobs
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.listobs')
  listobs(vis = 'uid___A002_X91cc20_X6d8.ms',
    listfile = 'uid___A002_X91cc20_X6d8.ms.listobs')
  
  

# A priori flagging
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_X91cc20_X6d8.ms',
    mode = 'manual',
    spw = '1~20',
    autocorr = T,
    flagbackup = F)
  
  flagdata(vis = 'uid___A002_X91cc20_X6d8.ms',
    mode = 'manual',
    intent = '*POINTING*,*ATMOSPHERE*',
    flagbackup = F)
  
  flagcmd(vis = 'uid___A002_X91cc20_X6d8.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_X91cc20_X6d8.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_X91cc20_X6d8.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')
  

# Generation and time averaging of the WVR cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.wvr') 
  
  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.wvrgcal') 
  
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X91cc20_X6d8.ms.wvrgcal')
  
  wvrgcal(vis = 'uid___A002_X91cc20_X6d8.ms',
    caltable = 'uid___A002_X91cc20_X6d8.ms.wvr',
    toffset = 0,
    tie = ['HL_Tau,J0431+1731'],
    statsource = 'HL_Tau')
  
  casalog.setlogfile(mylogfile)
  
  # This is a temporary workaround, which will be included in a future version of CASA
  
  tb.open('uid___A002_X91cc20_X6d8.ms.wvr', nomodify=False)
  count = 0
  numrows = tb.nrows()
  mycparamcol = tb.getcol('CPARAM')
  for i in range(0, numrows):
      if mycparamcol[0][0][i] == (1.+0.j):
          tb.putcell('FLAG', i, [[True]])
          count += 1
  tb.close()
  del mycparamcol
  if(numrows>0):
      print 'Flagged', count, 'of', numrows, 'solutions =', 100.*count/float(numrows),'%'
  
  
  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.wvr.smooth') 
  
  smoothcal(vis = 'uid___A002_X91cc20_X6d8.ms',
    tablein = 'uid___A002_X91cc20_X6d8.ms.wvr',
    caltable = 'uid___A002_X91cc20_X6d8.ms.wvr.smooth',
    smoothtype = 'mean',
    smoothtime = 2.016)
  
  
  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_X91cc20_X6d8.ms.wvr.smooth', spw='9', antenna='DA51',
    yrange=[-199,199],subplot=22, interactive=False,
    figfile='uid___A002_X91cc20_X6d8.ms.wvr.smooth.plots/uid___A002_X91cc20_X6d8.ms.wvr.smooth') 
  
  #Note: If you see wraps in these plots, try changing yrange or unwrap=True 
  #Note: If all plots look strange, it may be a bad WVR on the reference antenna.
  #      To check, you can set antenna='' to show all baselines.
  

# Generation of the Tsys cal table
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.tsys') 
  gencal(vis = 'uid___A002_X91cc20_X6d8.ms',
    caltable = 'uid___A002_X91cc20_X6d8.ms.tsys',
    caltype = 'tsys')

  flagdata('uid___A002_X91cc20_X6d8.ms.tsys',
           spw='9:0~3;124~127,11:0~3;124~127,13:0~3;124~127,15:0~3;124~127')
  
  if applyonly != True: aU.plotbandpass(caltable='uid___A002_X91cc20_X6d8.ms.tsys', overlay='time', 
    xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
    showatm=True,pwv='auto',chanrange='5~123',showfdm=True, 
    field='', figfile='uid___A002_X91cc20_X6d8.ms.tsys.plots.overlayTime/uid___A002_X91cc20_X6d8.ms.tsys') 
  
  
  if applyonly != True: es.checkCalTable('uid___A002_X91cc20_X6d8.ms.tsys', msName='uid___A002_X91cc20_X6d8.ms', interactive=False) 
  

# Generation of the antenna position cal table
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.antpos') 
  gencal(vis = 'uid___A002_X91cc20_X6d8.ms',
    caltable = 'uid___A002_X91cc20_X6d8.ms.antpos',
    caltype = 'antpos',
    antenna = 'DV25',
    parameter = [0.00176184759091,-0.00114033182566,-0.00248903422913])

  

# Application of the WVR, Tsys and antpos cal tables
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  
  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_X91cc20_X6d8.ms', tsystable = 'uid___A002_X91cc20_X6d8.ms.tsys', tsysChanTol = 1)
  
  
  
  applycal(vis = 'uid___A002_X91cc20_X6d8.ms',
    field = '0',
    spw = '9,11,13,15',
    gaintable = ['uid___A002_X91cc20_X6d8.ms.tsys', 'uid___A002_X91cc20_X6d8.ms.wvr.smooth', 'uid___A002_X91cc20_X6d8.ms.antpos'],
    gainfield = ['0', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X91cc20_X6d8.ms',
    field = '1',
    spw = '9,11,13,15',
    gaintable = ['uid___A002_X91cc20_X6d8.ms.tsys', 'uid___A002_X91cc20_X6d8.ms.wvr.smooth', 'uid___A002_X91cc20_X6d8.ms.antpos'],
    gainfield = ['1', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X91cc20_X6d8.ms',
    field = '2',
    spw = '9,11,13,15',
    gaintable = ['uid___A002_X91cc20_X6d8.ms.tsys', 'uid___A002_X91cc20_X6d8.ms.wvr.smooth', 'uid___A002_X91cc20_X6d8.ms.antpos'],
    gainfield = ['2', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X91cc20_X6d8.ms',
    field = '3',
    spw = '9,11,13,15',
    gaintable = ['uid___A002_X91cc20_X6d8.ms.tsys', 'uid___A002_X91cc20_X6d8.ms.wvr.smooth', 'uid___A002_X91cc20_X6d8.ms.antpos'],
    gainfield = ['3', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  if applyonly != True: es.getCalWeightStats('uid___A002_X91cc20_X6d8.ms') 
  

# Split out science SPWs and time average
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.split') 
  split(vis = 'uid___A002_X91cc20_X6d8.ms',
    outputvis = 'uid___A002_X91cc20_X6d8.ms.split',
    datacolumn = 'corrected',
    spw = '9,11,13,15',
    keepflags = T)
  
  

print "# Calibration"

# Listobs, clear pointing table, and save original flags
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.split.listobs')
  listobs(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    listfile = 'uid___A002_X91cc20_X6d8.ms.split.listobs')
  
  tb.open('uid___A002_X91cc20_X6d8.ms.split/POINTING', nomodify = False)
  a = tb.rownumbers()
  tb.removerows(a)
  tb.close()
  
  if not os.path.exists('uid___A002_X91cc20_X6d8.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_X91cc20_X6d8.ms.split',
      mode = 'save',
      versionname = 'Original')
  
  

# Initial flagging
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    mode = 'shadow',
    flagbackup = F)
  
  # Flagging edge channels
  
  flagdata(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    mode = 'manual',
    spw = '0:0~7;120~127,1:0~7;120~127,2:0~7;120~127,3:0~7;120~127',
    flagbackup = F)

  # manual flags
  flagdata(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    mode = 'manual',
    antenna='DV06,DA42',
    flagbackup = F)

  flagdata(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    mode = 'manual',
    timerange='05:49:08.1~05:49:12.3',
    flagbackup = F)

  flagdata(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    mode = 'manual',
    antenna='DA45',spw='2,3',
    flagbackup = F)

  flagdata(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    mode = 'manual',
    antenna='DV14',spw='3',
    flagbackup = F)  
  
  

# Putting a model for the flux calibrator(s)
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  setjy('uid___A002_X91cc20_X6d8.ms.split',
        standard='manual', field='J0423-0120', spix=-0.677913,
        reffreq='343.478950GHz', fluxdensity=[0.749364,0,0,0])



  
  

# Save flags before bandpass cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  

# Bandpass calibration
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.split.ap_pre_bandpass') 
  
  gaincal(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    caltable = 'uid___A002_X91cc20_X6d8.ms.split.ap_pre_bandpass',
    field = '0', # J0423-0120
    spw = '0:51~76,1:51~76,2:51~76,3:51~76',
    solint = 'int',
    refant = 'DA51',
    calmode = 'p')
  
  if applyonly != True: es.checkCalTable('uid___A002_X91cc20_X6d8.ms.split.ap_pre_bandpass', msName='uid___A002_X91cc20_X6d8.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.split.bandpass') 
  bandpass(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    caltable = 'uid___A002_X91cc20_X6d8.ms.split.bandpass',
    field = '0', # J0423-0120
    solint = 'inf',
    combine = 'scan',
    refant = 'DA51',
    solnorm = True,
    bandtype = 'B',
    gaintable = 'uid___A002_X91cc20_X6d8.ms.split.ap_pre_bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X91cc20_X6d8.ms.split.bandpass', msName='uid___A002_X91cc20_X6d8.ms.split', interactive=False) 
  

# Save flags before gain cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.split.phase_int') 
  gaincal(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    caltable = 'uid___A002_X91cc20_X6d8.ms.split.phase_int',
    field = '0~2', # J0423-0120,J0510+1800,J0431+1731
    solint = 'int',
    refant = 'DA51',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X91cc20_X6d8.ms.split.bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X91cc20_X6d8.ms.split.phase_int', msName='uid___A002_X91cc20_X6d8.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.split.ampli_inf') 
  gaincal(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    caltable = 'uid___A002_X91cc20_X6d8.ms.split.ampli_inf',
    field = '0~2', # J0423-0120,J0510+1800,J0431+1731
    solint = 'inf',
    refant = 'DA51',
    gaintype = 'T',
    calmode = 'a',
    gaintable = ['uid___A002_X91cc20_X6d8.ms.split.bandpass', 'uid___A002_X91cc20_X6d8.ms.split.phase_int'])
  
  if applyonly != True: es.checkCalTable('uid___A002_X91cc20_X6d8.ms.split.ampli_inf', msName='uid___A002_X91cc20_X6d8.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.split.flux_inf') 
  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.split.fluxscale') 
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X91cc20_X6d8.ms.split.fluxscale')
  
  fluxscaleDict = fluxscale(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    caltable = 'uid___A002_X91cc20_X6d8.ms.split.ampli_inf',
    fluxtable = 'uid___A002_X91cc20_X6d8.ms.split.flux_inf',
    reference = '0') # J0423-0120
  
  casalog.setlogfile(mylogfile)
  
  if applyonly != True: es.fluxscale2(caltable = 'uid___A002_X91cc20_X6d8.ms.split.ampli_inf', removeOutliers=True, msName='uid___A002_X91cc20_X6d8.ms', writeToFile=True, preavg=10000)
  
  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.split.phase_inf') 
  gaincal(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    caltable = 'uid___A002_X91cc20_X6d8.ms.split.phase_inf',
    field = '0~2', # J0423-0120,J0510+1800,J0431+1731
    solint = 'inf',
    refant = 'DA51',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X91cc20_X6d8.ms.split.bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X91cc20_X6d8.ms.split.phase_inf', msName='uid___A002_X91cc20_X6d8.ms.split', interactive=False) 
  

# Save flags before applycal
mystep = 16
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 17
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['0', '1']: # J0423-0120,J0510+1800
    applycal(vis = 'uid___A002_X91cc20_X6d8.ms.split',
      field = str(i),
      gaintable = ['uid___A002_X91cc20_X6d8.ms.split.bandpass', 'uid___A002_X91cc20_X6d8.ms.split.phase_int', 'uid___A002_X91cc20_X6d8.ms.split.flux_inf'],
      gainfield = ['', i, i],
      interp = 'linear,linear',
      calwt = F,
      flagbackup = F)
  
  applycal(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    field = '2,3', # HL_Tau
    gaintable = ['uid___A002_X91cc20_X6d8.ms.split.bandpass', 'uid___A002_X91cc20_X6d8.ms.split.phase_inf', 'uid___A002_X91cc20_X6d8.ms.split.flux_inf'],
    gainfield = ['', '2', '2'], # J0431+1731
    interp = 'linear,linear',
    calwt = F,
    flagbackup = F)
  

# Split out corrected column
mystep = 18
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X91cc20_X6d8.ms.split.cal') 
  split(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    outputvis = 'uid___A002_X91cc20_X6d8.ms.split.cal',
    datacolumn = 'corrected',
    keepflags = T)
  
  

  os.system('rm -rf HLTau_X6d8.ms.split.cal') 
  split(vis = 'uid___A002_X91cc20_X6d8.ms.split',
    outputvis = 'HLTau_X6d8.ms.split.cal',
    datacolumn = 'corrected',field='3',
    keepflags = T)
